// C (no pointers) Practicum Unit Tests
// SWEN-250
// Caity Kurutz, under the guidance of Christian Newman
// 2/12/2024

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "c_practicum_cdk.h"
#include "unit_tests.h"

// Simple boolean assert function for unit testing
// Author: Larry Kiser
int assert( int test_result, char error_format[], ... ) {
    va_list arguments ;
    static int test_number = 1 ;
    int result = 1 ;    // return 1 for test passed or 0 if failed
    
    if ( ! test_result ) {
        va_start( arguments, error_format ) ;
        printf( "Test # %d failed: ", test_number ) ;
        vprintf( error_format, arguments ) ;
        printf( "\n" ) ;
        result = 0 ;
    }
    test_number++ ;
    return result ;
}

int test()
{
    int passcount = 0 ;
    int failcount = 0 ;
    int result ;
    
    // Tests for fix_bad_code function
    printf( "\n----------------------- Test the fix_bad_code function -------------------------\n" ) ;

    //Test 1
    char to[5];
    char from[] = "epic";
    result = fix_bad_code(from, to);

    assert(result == 0,
    "fix_bad_code needs to return 0 using strcmp when two strings are equivalent, meaning that one was put into the other, but you got: %d", result)
    ? passcount++ : failcount++ ;

    //Test 2, just incase a student manually returns 0
    assert(strcmp(from, to) == 0,
    "The string is supposed to be %s (from), but you got: %s (to).", from, to)
    ? passcount++ : failcount++ ;


    printf( "\n----------------------- Test the count_periods function -------------------------\n" ) ;

    //Test 1
    char test[] = "this has no periods in it";
    result = count_periods(test) ;
    assert(result == 0,
    "count_periods needs to return 0 when there are no periods in the string, but you got: %d", result)
    ? passcount++ : failcount++ ;

    //Test 2
    char test2[] = "thi.s has peri.ods in it...";
    result = count_periods(test2) ;
    assert(result == 5,
    "count_periods needs to return 5 for the string '%s', but you got: %d", test2, result)
    ? passcount++ : failcount++ ;

    //Test 3
    char test3[] ="";
    result = count_periods(test3) ;
    assert(result == -1,
    "count_periods needs to return -1 when the passed string is empty, but you got: %d", result)
    ? passcount++ : failcount++ ;


    printf( "\n----------------------- Test the replace_colons_at_index function -------------------------\n" ) ;

    //Test 1
    char colon_test[] = "this h:as so many c:olons in :it:";
    char colon_expected[] = "this h;as so many c:olons in :it:";
    result = replace_colons_at_index(colon_test, 6) ;

    assert(strcmp(colon_test, colon_expected) == 0,
    "You got '%s' after replacing, but '%s' was expected.", colon_test, colon_expected)
    ? passcount++ : failcount++ ;

    assert(result == 1,
    "You returned %d, but %d was expected.", result, 1)
    ? passcount++ : failcount++ ;
    
    assert(strcmp(colon_test, colon_expected) == 0,
    "You got: %s, but %s was expected.", colon_test, colon_expected)
    ? passcount++ : failcount++ ;

    //Test 2
    char colon_test_2[] = "";
    result = replace_colons_at_index(colon_test_2, 6);
    assert(result == 0,
    "You returned %d, but %d was expected. because the string was empty.", result, 0)
    ? passcount++ : failcount++ ;

    //Test 3
    char colon_test_3[] = "word";
    result = replace_colons_at_index(colon_test_3, 0);
    assert(result == -1,
    "You returned %d, but %d was expected. because the string did not have a semi-colon at that index.", result, 0)
    ? passcount++ : failcount++ ;


    printf( "\nSummary of unit tests:\n%d tests passed\n%d tests failed\n\n", passcount, failcount ) ;

    return failcount ;
}